﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Scheduling.MRCPSP {

  [Item(nameof(ResourceCapacity), "")]
  [StorableClass]
  public sealed class ResourceCapacity : ParameterizedNamedItem {
    //public ResourceCapacity()
    //  : this() {
    //}

    [Storable]
    private readonly IFixedValueParameter<IntValue> capacityParam;

    [Storable]
    private readonly IFixedValueParameter<BoolValue> isRenewableParam;

    [Storable]
    private readonly IFixedValueParameter<IntValue> numberParam;

    public ResourceCapacity() {
      Parameters.Add(numberParam = new FixedValueParameter<IntValue>(nameof(Number), new IntValue()));
      Parameters.Add(capacityParam = new FixedValueParameter<IntValue>(nameof(Capacity), new IntValue()));
      Parameters.Add(isRenewableParam = new FixedValueParameter<BoolValue>(nameof(IsRenewable), new BoolValue()));
    }

    [StorableConstructor]
    private ResourceCapacity(bool deserializing)
      : base(deserializing) { }

    public override string ToString() => $"{Name} (Resource {Number})";

    private ResourceCapacity(ResourceCapacity original, Cloner cloner)
      : base(original, cloner) {
      numberParam = cloner.Clone(original.numberParam);
      capacityParam = cloner.Clone(original.capacityParam);
      isRenewableParam = cloner.Clone(original.isRenewableParam);
    }

    public int Capacity {
      get => capacityParam.Value.Value;
      set => capacityParam.Value.Value = value;
    }

    public bool IsRenewable {
      get => isRenewableParam.Value.Value;
      set => isRenewableParam.Value.Value = value;
    }

    public int Number {
      get => numberParam.Value.Value;
      set => numberParam.Value.Value = value;
    }

    public override IDeepCloneable Clone(Cloner cloner) => new ResourceCapacity(this, cloner);
  }
}
